/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "destructible.h"

//Ctor
Destructible::Destructible()
{}

///Finds and registers all destroyable objects in scene
void Destructible::RegisterDestructibleObjects(dtCore::Map &map)
{
	dtCore::RefPtr<dtCore::Object> temp;
	
	typedef std::vector<dtCore::RefPtr<dtCore::ActorProxy>> ProxyVector;
	ProxyVector proxies;
	map.FindProxies(proxies, "*");
	ProxyVector::iterator end_iter = proxies.end();

	//Go through the proxies
	for (ProxyVector::iterator iter = proxies.begin(); iter != end_iter; ++iter)
	{
		//Find ones that can be destroyed
		if ((*iter)->GetName().substr(0, 2) == "D_")
		{
			//Set their physical properties
			dMass *mass = new dMass();
			temp = dynamic_cast<dtCore::Object*>((*iter)->GetActor());
			temp->EnableDynamics(true);
			temp->GetBodyWrapper()->SetLinearDamping(0.001);
			temp->GetBodyWrapper()->SetAngularDamping(0.001);
			temp->GetMass(mass);

			//Set their inertia tensor, so they rotate naturally after collision
			float inert = mass->mass * mass->mass;
			mass->setParameters(mass->mass,
				mass->c[0], mass->c[1], mass->c[2],
				inert,inert, inert,
				0.0f, 0.0f, 0.0f); 
			temp->SetMass(mass);

			//Register object
			DestructibleObject obj;
			obj.Health = temp->GetMass() * 5.0;
			obj.Object = temp;
			DestructibleObjectBuffer.push_back(obj);
		}
	}
}

///Damage object in scene for given amount
///Returns true if objects health goes below 0.
bool Destructible::DamageObject(double damage, dtCore::Transformable* damagedObject, dtCore::Scene *scene)
{
	if(damagedObject == NULL)
		return false;

	//Cache name
	std::string name = damagedObject->GetName();

	//Find correct object and damage it
	for(int i = DestructibleObjectBuffer.size() - 1; i >= 0; i--)
	{
		if(DestructibleObjectBuffer[i].Object->GetName() == name)
		{
			DestructibleObjectBuffer[i].Health -= damage;

			//Check object destroyed
			if(DestructibleObjectBuffer[i].Health < 0)
			{
				
				//Add explosion if object is destroyed
				dtCore::Transform trans;
				DestructibleObjectBuffer[i].Object->GetTransform(trans);
				dtCore::EffectManager::GetInstance(0)->AddDetonation(trans.GetTranslation(), "Explosion", 1.0f);
				Sound::PlayExplosion()->SetPosition(trans.GetTranslation());
				
				DestructibleObjectBuffer.erase(DestructibleObjectBuffer.begin() + i);

				return true;
			}
			break;
		}
	}
	return false;
}