/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#pragma once
#define dSINGLE

//Delta
#include <dtABC/application.h>
#include <dtABC/proximitytrigger.h>
#include <dtCore/actorproxy.h>
#include <dtCore/base.h>
#include <dtCore/deltawin.h>
#include <dtCore/effectmanager.h>
#include <dtCore/exceptionenum.h>
#include <dtABC/labelactor.h>
#include <dtCore/map.h>
#include <dtCore/mouse.h>
#include <dtCore/odecontroller.h>
#include <dtCore/odespacewrap.h>
#include <dtCore/object.h>
#include <dtCore/project.h>
#include <dtCore/refptr.h>
#include <dtCore/scene.h>
#include <dtCore/system.h>
#include <dtCore/transform.h>
#include <dtGame/gamemanager.h>
#include <dtGUI/gui.h>
#include <dtGUI/scriptmodule.h>
#include <dtUtil/log.h>
#include <dtUtil/datapathutils.h>
#include <ode/collision.h>
#include <ode/objects.h>

//CEGUI
#include <CEGUI/CEGUIWindow.h>
#include <CEGUI/CEGUIExceptions.h>
#include <CEGUI/CEGUIDefaultResourceProvider.h>

//C++
#include <cstdio>

//Libs
#include "json/json-forwards.h"
#include "json/json.h"

//Game
#include "gui.h"
#include "player.h"
#include "level.h"
#include "shooter.h"
#include "sound.h"
#include "destructible.h"

///Callback for collision handling
void NearCollisionCallback (void *data, dGeomID o1, dGeomID o2);

class Game : public dtABC::Application
{
//Variables
private:
	Json::Value                           m_Save;
	Json::Value                           m_Config;

	Gui                                  *m_Gui;
	Player                               *m_Player;
	Level                                *m_Level;
	Destructible                         *m_Destructible;

	bool		                          m_IsMainMenuLoaded;
    dtCore::RefPtr<dtCore::EffectManager> m_EffectManager;
	dNearCallback                        *m_defaultCollisionCallback;
	std::vector<dtCore::Transformable*>   m_ObjectsToRemove;


//Methods
public:
	Game(const std::string& configFilename = "config.xml");
	~Game(void);

	void Game::Config();
	bool Game::KeyPressed(const dtCore::Keyboard* keyboard, int key );
	bool Game::KeyReleased(const dtCore::Keyboard* keyboard, int key );
	bool Game::MouseMoved (const dtCore::Mouse *mouse, float x, float y);
    bool Game::MouseDragged(const dtCore::Mouse* mouse, float x, float y);
	bool Game::MouseScrolled(const dtCore::Mouse* mouse, int delta);
    bool Game::MouseButtonPressed(const dtCore::Mouse* mouse, dtCore::Mouse::MouseButton button);
	void Game::OnMessage(MessageData* data);
	void Game::Frame(const double deltaSimTime);
	void Game::PreFrame(const double deltaSimTime);
	void Game::PostFrame(const double deltaSimTime);
	void Game::ProcessShot(dtCore::Transformable * shot, dtCore::Transformable * target);
	void Game::ProcessCollision(dtCore::Scene::CollisionData cd);

private:
	void Game::CreateHelp();
	void Game::LoadConfig();
	void Game::LoadSave();
	void Game::SaveConfig();
	void Game::SaveSave();
	void Game::LoadMission(MISSION mission);
	void Game::NewSave();
	void Game::NewConfig();
	void Game::LoadMainMenuScene();
	void Game::SetDefaultWindow();

};

