/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "gui.h"

Gui::Gui()
{}

///Constructor
Gui::Gui(dtCore::Camera *cam, dtCore::Keyboard *keyboard, dtCore::Mouse *mouse, Json::Value *config)
{
	//Store pointer to config, so we can edit settings
	m_Config = config;
	m_GUI = new dtGUI::GUI(cam, keyboard, mouse);

	//Try to load gui from files
	try
	{
		dtGUI::GUI::SetDefaultParser("XercesParser");

		m_GUI->LoadScheme("WindowsLook.scheme");

		///Create ScriptModule which will handle callbacks to widgets when it loads the Layout file.
		dtGUI::ScriptModule* sm = new dtGUI::ScriptModule();
		sm->AddCallback("ContinueHandler"            , CEGUI::SubscriberSlot(&Gui::ContinueHandler, this));
		sm->AddCallback("NewHandler"                 , CEGUI::SubscriberSlot(&Gui::NewGameHandler, this));
		sm->AddCallback("SettingsHandler"            , CEGUI::SubscriberSlot(&Gui::SettingsHandler, this));
		sm->AddCallback("QuitHandler"                , CEGUI::SubscriberSlot(&Gui::QuitHandler, this));
		
		sm->AddCallback("StoryAdvanceHandler"        , CEGUI::SubscriberSlot(&Gui::StoryOkHandler, this));
		
		sm->AddCallback("SettingsOkHandler"          , CEGUI::SubscriberSlot(&Gui::SettingsOkHandler, this));
		sm->AddCallback("SettingsHelpChanged"        , CEGUI::SubscriberSlot(&Gui::SettingsHelpChanged, this));
		sm->AddCallback("SettingsFulllscreenChanged" , CEGUI::SubscriberSlot(&Gui::SettingsFulllscreenChanged, this));
		sm->AddCallback("SettingsInvertXChanged"     , CEGUI::SubscriberSlot(&Gui::SettingsInvertXChanged, this));
		sm->AddCallback("SettingsInvertYChanged"     , CEGUI::SubscriberSlot(&Gui::SettingsInvertYChanged, this));

		CEGUI::System::getSingleton().setScriptingModule(sm);

		//Load GUI layout from file
		m_GUI->LoadLayout("game.layout");

		//Store pointers to main widgets
		m_Menu     = m_GUI->GetWidget("Menu");
		m_Settings = m_GUI->GetWidget("SettingsGroup");
		m_Hud      = m_GUI->GetWidget("HUD");
		m_Story    = m_GUI->GetWidget("Story");
		m_Help     = m_GUI->GetWidget("HelpText");
			
		//Set some default values
		m_GUI->GetWidget("SettingsBackground")->disable();
		((CEGUI::Window*)m_Story)->setAlwaysOnTop(true);
		
		//Hide things we dont want to see when main menu is loaded
		m_Hud->hide();
		m_Story->hide();
		m_Settings->hide();
		
		//Pass through mouse events so player can move when gui with information about ship is displayed
		m_GUI->GetWidget("Root")->setMousePassThroughEnabled(true);
		m_Hud->setMousePassThroughEnabled(true);
		
		//Load those parts of gui, that are often accessed
		//EngineFull
		m_EngineFull00  = m_GUI->GetWidget("EngineFull00");
		m_EngineFull01  = m_GUI->GetWidget("EngineFull01"); 
		m_EngineFull02  = m_GUI->GetWidget("EngineFull02"); 
		m_EngineFull03  = m_GUI->GetWidget("EngineFull03"); 
		m_EngineFull04  = m_GUI->GetWidget("EngineFull04"); 
		m_EngineFull05  = m_GUI->GetWidget("EngineFull05"); 
		m_EngineFull06  = m_GUI->GetWidget("EngineFull06"); 
		m_EngineFull07  = m_GUI->GetWidget("EngineFull07"); 
		m_EngineFull08  = m_GUI->GetWidget("EngineFull08"); 
		m_EngineFull09  = m_GUI->GetWidget("EngineFull09"); 
		m_EngineFull10  = m_GUI->GetWidget("EngineFull10"); 

		//EngineEmpty
		m_EngineEmpty00 = m_GUI->GetWidget("EngineEmpty00"); 
		m_EngineEmpty01 = m_GUI->GetWidget("EngineEmpty01"); 
		m_EngineEmpty02 = m_GUI->GetWidget("EngineEmpty02"); 
		m_EngineEmpty03 = m_GUI->GetWidget("EngineEmpty03"); 
		m_EngineEmpty04 = m_GUI->GetWidget("EngineEmpty04"); 
		m_EngineEmpty05 = m_GUI->GetWidget("EngineEmpty05"); 
		m_EngineEmpty06 = m_GUI->GetWidget("EngineEmpty06"); 
		m_EngineEmpty07 = m_GUI->GetWidget("EngineEmpty07"); 
		m_EngineEmpty08 = m_GUI->GetWidget("EngineEmpty08"); 
		m_EngineEmpty09 = m_GUI->GetWidget("EngineEmpty09"); 
		m_EngineEmpty10 = m_GUI->GetWidget("EngineEmpty10"); 

		//HullFull
		m_HullFull00    = m_GUI->GetWidget("HullFull0"); 
		m_HullFull01    = m_GUI->GetWidget("HullFull1"); 
		m_HullFull02    = m_GUI->GetWidget("HullFull2"); 
		m_HullFull03    = m_GUI->GetWidget("HullFull3"); 
		m_HullFull04    = m_GUI->GetWidget("HullFull4"); 
		m_HullFull05    = m_GUI->GetWidget("HullFull5"); 
		m_HullFull06    = m_GUI->GetWidget("HullFull6"); 
		m_HullFull07    = m_GUI->GetWidget("HullFull7"); 
		m_HullFull08    = m_GUI->GetWidget("HullFull8"); 
		m_HullFull09    = m_GUI->GetWidget("HullFull9"); 

		//HullEmpty
		m_HullEmpty00   = m_GUI->GetWidget("HullEmpty0"); 
		m_HullEmpty01   = m_GUI->GetWidget("HullEmpty1"); 
		m_HullEmpty02   = m_GUI->GetWidget("HullEmpty2"); 
		m_HullEmpty03   = m_GUI->GetWidget("HullEmpty3"); 
		m_HullEmpty04   = m_GUI->GetWidget("HullEmpty4"); 
		m_HullEmpty05   = m_GUI->GetWidget("HullEmpty5"); 
		m_HullEmpty06   = m_GUI->GetWidget("HullEmpty6"); 
		m_HullEmpty07   = m_GUI->GetWidget("HullEmpty7"); 
		m_HullEmpty08   = m_GUI->GetWidget("HullEmpty8"); 
		m_HullEmpty09   = m_GUI->GetWidget("HullEmpty9");

		//ShieldFull
		m_ShieldFull00  = m_GUI->GetWidget("ShieldFull0"); 
		m_ShieldFull01  = m_GUI->GetWidget("ShieldFull1"); 
		m_ShieldFull02  = m_GUI->GetWidget("ShieldFull2"); 
		m_ShieldFull03  = m_GUI->GetWidget("ShieldFull3"); 
		m_ShieldFull04  = m_GUI->GetWidget("ShieldFull4"); 
		m_ShieldFull05  = m_GUI->GetWidget("ShieldFull5"); 
		m_ShieldFull06  = m_GUI->GetWidget("ShieldFull6"); 
		m_ShieldFull07  = m_GUI->GetWidget("ShieldFull7"); 
		m_ShieldFull08  = m_GUI->GetWidget("ShieldFull8"); 
		m_ShieldFull09  = m_GUI->GetWidget("ShieldFull9");

		//ShieldEmpty
		m_ShieldEmpty00 = m_GUI->GetWidget("ShieldEmpty0"); 
		m_ShieldEmpty01 = m_GUI->GetWidget("ShieldEmpty1"); 
		m_ShieldEmpty02 = m_GUI->GetWidget("ShieldEmpty2"); 
		m_ShieldEmpty03 = m_GUI->GetWidget("ShieldEmpty3"); 
		m_ShieldEmpty04 = m_GUI->GetWidget("ShieldEmpty4"); 
		m_ShieldEmpty05 = m_GUI->GetWidget("ShieldEmpty5"); 
		m_ShieldEmpty06 = m_GUI->GetWidget("ShieldEmpty6"); 
		m_ShieldEmpty07 = m_GUI->GetWidget("ShieldEmpty7"); 
		m_ShieldEmpty08 = m_GUI->GetWidget("ShieldEmpty8"); 
		m_ShieldEmpty09 = m_GUI->GetWidget("ShieldEmpty9"); 
	}

	// Log all errors
	catch(CEGUI::Exception& e)
	{
		std::cout << "E:CEGUI: " << e.getMessage().c_str() << std::endl;
	}
}

///Destructor
Gui::~Gui(void)
{
	CEGUI::ScriptModule* sm = CEGUI::System::getSingleton().getScriptingModule();
	if (sm)
	{
		//Clear GUI
		CEGUI::System::getSingleton().setScriptingModule(NULL);

		delete sm;
	}
}

///Show main menu and hide hud.
void Gui::Show()
{
	m_Menu->show();
	m_Hud->hide();
}

///Show hud and hide main menu.
void Gui::Hide()
{
	m_Menu->hide();
	m_Hud->show();
}

///Return if main menu is visible.
void Gui::IsVisible(bool visibility)
{
	m_Menu->setVisible(visibility);
	m_Settings->setVisible(m_Settings->isVisible() && visibility);
	m_Hud->setVisible(!visibility);
	m_Menu->enable();
}

///Set if main menu is visible.
bool Gui::IsVisible()
{
	if(m_Menu != 0)
	{
		return m_Menu->isVisible();
	}
	return false;
}

///Action called when new game button is pressed.
bool Gui::ContinueHandler(const CEGUI::EventArgs& e)
{
	Sound::GetInstance()->PlayMenuButton();
	Event * event = new Event(Gui::ButtonEvent::CONTINUE_GAME);
	SendMessage("GUI", event);
	ContinueEnabled(false);
    return true;
}

///Action called when new game button is pressed.
bool Gui::NewGameHandler(const CEGUI::EventArgs& e)
{
	Sound::GetInstance()->PlayMenuButton();
	Event * event = new Event(Gui::ButtonEvent::NEW_GAME);
	SendMessage("GUI", event);
	dtCore::System::GetInstance(0)->SetPause(false);
	ContinueEnabled(false);
    return true;
}

///Action called when quit game button is pressed.
bool Gui::QuitHandler(const CEGUI::EventArgs& e)
{
	Sound::GetInstance()->PlayMenuButton();
	Event * event = new Event(Gui::ButtonEvent::EXIT_GAME);
	SendMessage("GUI", event);
    return true;
}

///Action called when button in story window iss pressed.
bool Gui::StoryOkHandler(const CEGUI::EventArgs& e)
{
	//Restore visibility of cursor
	dtABC::Application::GetInstance(0)->GetWindow()->SetShowCursor(m_WasCursorVisible);
	
	//Restore time scale
	dtCore::System::GetInstance().SetTimeScale(1.0);

	//Play sound
	Sound::GetInstance()->PlayMenuButton();

	//Hide window
	m_Story->hide();

	//Send event to game
	Event * event = new Event(Gui::ButtonEvent::STORY_ADVANCE);
	SendMessage("GUI", event);

    return true;
}

///Action called when settings button is pressed.
bool Gui::SettingsHandler(const CEGUI::EventArgs& e)
{
	Sound::GetInstance()->PlayMenuButton();
	m_Menu->disable();
	((CEGUI::Checkbox*)(m_Settings->getChild("SettingsHelp")))->setSelected((*m_Config)["ShowHelp"].asBool());
	((CEGUI::Checkbox*)(m_Settings->getChild("SettingsFulllscreen")))->setSelected((*m_Config)["Fullscreen"].asBool());
	((CEGUI::Checkbox*)(m_Settings->getChild("SettingsInvertX")))->setSelected((*m_Config)["InvertX"].asBool());
	((CEGUI::Checkbox*)(m_Settings->getChild("SettingsInvertY")))->setSelected((*m_Config)["InvertY"].asBool());
	m_Settings->setVisible(true);	
    return true;
}

///Action called when ok button in setting is pressed.
bool Gui::SettingsOkHandler(const CEGUI::EventArgs& e)
{
	Sound::GetInstance()->PlayMenuButton();
	m_Menu->enable();
	m_Settings->setVisible(false);
	Event * event = new Event(Gui::ButtonEvent::SETTINGS_OK);
	SendMessage("GUI", event);
    return true;
}

///Toggle for show help
bool Gui::SettingsHelpChanged(const CEGUI::EventArgs& e)
{
	if(m_Settings->isVisible() == false)
	{
		return false;
	}
	Sound::GetInstance()->PlayMenuButton();
	(*m_Config)["ShowHelp"] = !(*m_Config)["ShowHelp"].asBool();
	return true;
}

///Toggle for follscreen
bool Gui::SettingsFulllscreenChanged(const CEGUI::EventArgs& e)
{
	if(m_Settings->isVisible() == false)
	{
		return false;
	}
	Sound::GetInstance()->PlayMenuButton();
	(*m_Config)["Fullscreen"] = !(*m_Config)["Fullscreen"].asBool();
	return true;
}

///Toggle for invert x axis
bool Gui::SettingsInvertXChanged(const CEGUI::EventArgs& e)
{
	if(m_Settings->isVisible() == false)
	{
		return false;
	}
	Sound::GetInstance()->PlayMenuButton();
	(*m_Config)["InvertX"] = !(*m_Config)["InvertX"].asBool();
	return true;
}

///Toggle for invert y axis
bool Gui::SettingsInvertYChanged(const CEGUI::EventArgs& e)
{
	if(m_Settings->isVisible() == false)
	{
		return false;
	}
	Sound::GetInstance()->PlayMenuButton();
	(*m_Config)["InvertY"] = !(*m_Config)["InvertY"].asBool();
	return true;
}

///method called every update of game to update HUD
void Gui::UpdatePlayer(int Hull, int	Shield, double EnginePower)
{
	m_HullFull09 ->setVisible(Hull >  95);
	m_HullEmpty09->setVisible(Hull <= 95);
	m_HullFull08 ->setVisible(Hull >  85);
	m_HullEmpty08->setVisible(Hull <= 85);
	m_HullFull07 ->setVisible(Hull >  75);
	m_HullEmpty07->setVisible(Hull <= 75);
	m_HullFull06 ->setVisible(Hull >  65);
	m_HullEmpty06->setVisible(Hull <= 65);
	m_HullFull05 ->setVisible(Hull >  55);
	m_HullEmpty05->setVisible(Hull <= 55);
	m_HullFull04 ->setVisible(Hull >  45);
	m_HullEmpty04->setVisible(Hull <= 45);
	m_HullFull03 ->setVisible(Hull >  35);
	m_HullEmpty03->setVisible(Hull <= 35);
	m_HullFull02 ->setVisible(Hull >  25);
	m_HullEmpty02->setVisible(Hull <= 25);
	m_HullFull01 ->setVisible(Hull >  15);
	m_HullEmpty01->setVisible(Hull <= 15);
	m_HullFull00 ->setVisible(Hull >  5);
	m_HullEmpty00->setVisible(Hull <= 5);
	
	m_ShieldFull09 ->setVisible(Shield >  95);
	m_ShieldEmpty09->setVisible(Shield <= 95);
	m_ShieldFull08 ->setVisible(Shield >  85);
	m_ShieldEmpty08->setVisible(Shield <= 85);
	m_ShieldFull07 ->setVisible(Shield >  75);
	m_ShieldEmpty07->setVisible(Shield <= 75);
	m_ShieldFull06 ->setVisible(Shield >  65);
	m_ShieldEmpty06->setVisible(Shield <= 65);
	m_ShieldFull05 ->setVisible(Shield >  55);
	m_ShieldEmpty05->setVisible(Shield <= 55);
	m_ShieldFull04 ->setVisible(Shield >  45);
	m_ShieldEmpty04->setVisible(Shield <= 45);
	m_ShieldFull03 ->setVisible(Shield >  35);
	m_ShieldEmpty03->setVisible(Shield <= 35);
	m_ShieldFull02 ->setVisible(Shield >  25);
	m_ShieldEmpty02->setVisible(Shield <= 25);
	m_ShieldFull01 ->setVisible(Shield >  15);
	m_ShieldEmpty01->setVisible(Shield <= 15);
	m_ShieldFull00 ->setVisible(Shield >  5);
	m_ShieldEmpty00->setVisible(Shield <= 5);
	
	m_EngineFull10 ->setVisible(EnginePower >  0.95 || EnginePower < -0.01);
	m_EngineEmpty10->setVisible(m_EngineFull10->isVisible() == false);
	m_EngineFull09 ->setVisible(EnginePower >  0.95 || EnginePower < -0.01);
	m_EngineEmpty09->setVisible(m_EngineFull09->isVisible() == false);

	m_EngineFull08 ->setVisible(EnginePower >  0.75 || EnginePower < -0.11);
	m_EngineEmpty08->setVisible(m_EngineFull08->isVisible() == false);
	m_EngineFull07 ->setVisible(EnginePower >  0.75 || EnginePower < -0.11);
	m_EngineEmpty07->setVisible(m_EngineFull07->isVisible() == false);

	m_EngineFull06 ->setVisible(EnginePower >  0.55);
	m_EngineEmpty06->setVisible(EnginePower <= 0.55);
	m_EngineFull05 ->setVisible(EnginePower >  0.55);
	m_EngineEmpty05->setVisible(EnginePower <= 0.55);

	m_EngineFull04 ->setVisible(EnginePower >  0.35);
	m_EngineEmpty04->setVisible(EnginePower <= 0.35);
	m_EngineFull03 ->setVisible(EnginePower >  0.35);
	m_EngineEmpty03->setVisible(EnginePower <= 0.35);

	m_EngineFull02 ->setVisible(EnginePower >  0.15);
	m_EngineEmpty02->setVisible(EnginePower <= 0.15);
	m_EngineFull01 ->setVisible(EnginePower >  0.15);
	m_EngineEmpty01->setVisible(EnginePower <= 0.15);

	m_EngineFull00 ->setVisible(EnginePower >  0);
	m_EngineEmpty00->setVisible(EnginePower <= 0);


}

///Toggle visibility of text area with help
void Gui::ToggleHelp()
{
	m_Help->setVisible(!m_Help->isVisible());
	Sound::GetInstance()->PlayMenuButton();
}

///Set visibility of text area with help
void Gui::HelpVisible(bool value)
{
	m_Help->setVisible(value);
	Sound::GetInstance()->PlayMenuButton();
}

///Show poup with story text
void Gui::ShowStoryText(std::string text)
{
	//Slow down game
	dtCore::System::GetInstance().SetTimeScale(0.1);

	//Show cursos
	m_WasCursorVisible = dtABC::Application::GetInstance(0)->GetWindow()->GetShowCursor();
	dtABC::Application::GetInstance(0)->GetWindow()->SetShowCursor(true);

	//Show window
	m_Story->show();
	m_Story->getChild("StoryText")->setText(text);

	//Play sound
	Sound::GetInstance()->PlayMenuButton();
}

///Set whether continue button is enabled
void Gui::ContinueEnabled(bool enabled)
{
	if(enabled == true)
		m_Menu->getChild("ContinueGame")->enable();
	else
		m_Menu->getChild("ContinueGame")->disable();
}