/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#pragma once
#define dSINGLE //resolves fatal error C1004: unexpected end-of-file found

#include <cstdio>

#include <dtGUI/gui.h>
#include <dtABC/application.h>
#include <dtGUI/scriptmodule.h>
#include <CEGUI/CEGUIForwardRefs.h>
#include <CEGUI/CEGUI.h>
#include <CEGUI/CEGUIWindow.h>
#include <CEGUI/CEGUIExceptions.h>
#include <dtCore/project.h>
#include <dtCore/system.h>
#include <dtCore/deltawin.h>
#include <dtCore/base.h>

#include "sound.h"

#include "json/json-forwards.h"
#include "json/json.h"

class Gui: public dtCore::Base
{
//Definitions
public:
	//Possible actions sent by gui
	enum ButtonEvent
	{
		NONE,			//No or undefined action
		NEW_GAME,		//New game pressed
		SETTINGS_OK,	//Ok in settings pressed
		STORY_ADVANCE,	//Window with story text closed
		EXIT_GAME,	    //No explanation needed
		CONTINUE_GAME,  //User wants to continue prevoisly saved game
	};

	//Clas containing action in mesage sent by gui
	class Event
	{
	public:
		ButtonEvent Value;

		Event::Event(ButtonEvent value)
		{
			Value = value;
		}
	};

//Members
private:
	dtCore::RefPtr<dtGUI::GUI> m_GUI;	
	bool                       m_WasCursorVisible;
	Json::Value               *m_Config;

	dtGUI::Widget             *m_Menu;
	dtGUI::Widget             *m_Settings;
	dtGUI::Widget             *m_Hud;
	dtGUI::Widget             *m_Story;
	dtGUI::Widget             *m_Help;
	
	//Engine full
	dtGUI::Widget             *m_EngineFull00;
	dtGUI::Widget             *m_EngineFull01;
	dtGUI::Widget             *m_EngineFull02;
	dtGUI::Widget             *m_EngineFull03;
	dtGUI::Widget             *m_EngineFull04;
	dtGUI::Widget             *m_EngineFull05;
	dtGUI::Widget             *m_EngineFull06;
	dtGUI::Widget             *m_EngineFull07;
	dtGUI::Widget             *m_EngineFull08;
	dtGUI::Widget             *m_EngineFull09;
	dtGUI::Widget             *m_EngineFull10;
				              
	//Engine empty            
	dtGUI::Widget             *m_EngineEmpty00;
	dtGUI::Widget             *m_EngineEmpty01;
	dtGUI::Widget             *m_EngineEmpty02;
	dtGUI::Widget             *m_EngineEmpty03;
	dtGUI::Widget             *m_EngineEmpty04;
	dtGUI::Widget             *m_EngineEmpty05;
	dtGUI::Widget             *m_EngineEmpty06;
	dtGUI::Widget             *m_EngineEmpty07;
	dtGUI::Widget             *m_EngineEmpty08;
	dtGUI::Widget             *m_EngineEmpty09;
	dtGUI::Widget             *m_EngineEmpty10;
				              
	//Hull full	              
	dtGUI::Widget             *m_HullFull00;
	dtGUI::Widget             *m_HullFull01;
	dtGUI::Widget             *m_HullFull02;
	dtGUI::Widget             *m_HullFull03;
	dtGUI::Widget             *m_HullFull04;
	dtGUI::Widget             *m_HullFull05;
	dtGUI::Widget             *m_HullFull06;
	dtGUI::Widget             *m_HullFull07;
	dtGUI::Widget             *m_HullFull08;
	dtGUI::Widget             *m_HullFull09;
				              
	//Hull empty              
	dtGUI::Widget             *m_HullEmpty00;
	dtGUI::Widget             *m_HullEmpty01;
	dtGUI::Widget             *m_HullEmpty02;
	dtGUI::Widget             *m_HullEmpty03;
	dtGUI::Widget             *m_HullEmpty04;
	dtGUI::Widget             *m_HullEmpty05;
	dtGUI::Widget             *m_HullEmpty06;
	dtGUI::Widget             *m_HullEmpty07;
	dtGUI::Widget             *m_HullEmpty08;
	dtGUI::Widget             *m_HullEmpty09;
				              
	//Shield full             
	dtGUI::Widget             *m_ShieldFull00;
	dtGUI::Widget             *m_ShieldFull01;
	dtGUI::Widget             *m_ShieldFull02;
	dtGUI::Widget             *m_ShieldFull03;
	dtGUI::Widget             *m_ShieldFull04;
	dtGUI::Widget             *m_ShieldFull05;
	dtGUI::Widget             *m_ShieldFull06;
	dtGUI::Widget             *m_ShieldFull07;
	dtGUI::Widget             *m_ShieldFull08;
	dtGUI::Widget             *m_ShieldFull09;
				              
	//Shield empty            
	dtGUI::Widget             *m_ShieldEmpty00;
	dtGUI::Widget             *m_ShieldEmpty01;
	dtGUI::Widget             *m_ShieldEmpty02;
	dtGUI::Widget             *m_ShieldEmpty03;
	dtGUI::Widget             *m_ShieldEmpty04;
	dtGUI::Widget             *m_ShieldEmpty05;
	dtGUI::Widget             *m_ShieldEmpty06;
	dtGUI::Widget             *m_ShieldEmpty07;
	dtGUI::Widget             *m_ShieldEmpty08;
	dtGUI::Widget             *m_ShieldEmpty09;

//Methods
public:
	Gui::Gui(dtCore::Camera *cam, dtCore::Keyboard *keyboard, dtCore::Mouse *mouse, Json::Value *config);
	Gui::~Gui(void);
	void Gui::HelpVisible(bool value);
	bool Gui::IsVisible();
	void Gui::IsVisible(bool visibility);
	void Gui::Show();
	void Gui::Hide();
	void Gui::UpdatePlayer(int Hull, int	Shield, double EnginePower);
	void Gui::ToggleHelp();
	void Gui::ContinueEnabled(bool enabled);
	void Gui::ShowStoryText(std::string text);

private:
	Gui::Gui();
	bool Gui::QuitHandler(const CEGUI::EventArgs& e);
	bool Gui::SettingsHandler(const CEGUI::EventArgs& e);
	bool Gui::SettingsOkHandler(const CEGUI::EventArgs& e);
	bool Gui::NewGameHandler(const CEGUI::EventArgs& e);
	bool Gui::ContinueHandler(const CEGUI::EventArgs& e);
	bool Gui::StoryOkHandler(const CEGUI::EventArgs& e);
	bool Gui::SettingsHelpChanged(const CEGUI::EventArgs& e);
	bool Gui::SettingsFulllscreenChanged(const CEGUI::EventArgs& e);
	bool Gui::SettingsInvertXChanged(const CEGUI::EventArgs& e);
	bool Gui::SettingsInvertYChanged(const CEGUI::EventArgs& e);

};
