/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/


#include "level.h"

///Constructor
Level::Level(Json::Value * save, Gui * gui)
{
	m_Save     = save;
	m_Gui      = gui;
	m_WaitTime = 0.0;
	m_Wait     = false;
	PlayerDead = false;
	m_Player   = NULL;
}

///Defines dialogues and moves story forward when called
void Level::Advance()
{
	if(m_Save == NULL)
		return;
	if(m_Wait == true)
		return;	
	if(PlayerDead == true)
		return;
	
	//Pause check
	if(dtCore::System::GetInstance(0)->GetPause() == true)
	{
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 0.5;
		m_Wait = true;
		return;
	}

	//-----INTRODUCTION----------------
	if((*m_Save)["Dialog1"].asBool() == false)
	{
		(*m_Save)["Dialog1"] = true;
		m_Gui->ShowStoryText("Hi, this is flight commander speaking. So you finally got enough credits to buy a new ship?");
	}
	else if((*m_Save)["Dialog2"].asBool() == false)
	{
		(*m_Save)["Dialog2"] = true;
		m_Gui->ShowStoryText("It is my duty to inform you about controls of your ship.");
	}
	else if((*m_Save)["Dialog3"].asBool() == false)
	{
		(*m_Save)["Dialog3"] = true;
		m_Gui->ShowStoryText("I know you have been through basic flight course, but I have to do this anyway. The law is the law.");
	}
	//-----NAVIGATION TUTORIAL---------
	else if((*m_Save)["Dialog4"].asBool() == false)
	{
		(*m_Save)["Dialog4"] = true;
		m_Gui->ShowStoryText("So... as you know, you can rotate your ship by moving your mouse. Go on and try it.");
	}
	else if((*m_Save)["W4"].asBool() == false)
	{
		(*m_Save)["W4"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 5.0;
		m_Wait = true;
	}

	else if((*m_Save)["Dialog5"].asBool() == false)
	{
		(*m_Save)["Dialog5"] = true;
		m_Gui->ShowStoryText("Good. To accelerate or break, press R/F keys or scroll your mouse wheel.");
	}
	else if((*m_Save)["W5"].asBool() == false)
	{
		(*m_Save)["W5"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 10.0;
		m_Wait = true;
	}

	else if((*m_Save)["Dialog6"].asBool() == false)
	{
		(*m_Save)["Dialog6"] = true;
		m_Gui->ShowStoryText("Nice. Now try W/S/A/D keys.\nThey move your ship up/down/left and right.");
	}
	else if((*m_Save)["Dialog7"].asBool() == false)
	{
		(*m_Save)["Dialog7"] = true;
		m_Gui->ShowStoryText("Also with keys Q and E you can roll your ship.");
	}
	
	else if((*m_Save)["Dialog8"].asBool() == false)
	{
		(*m_Save)["Dialog8"] = true;
		m_Gui->ShowStoryText("And that is the basics for flying, try to fly around a little.");
	}
	else if((*m_Save)["W8"].asBool() == false)
	{
		(*m_Save)["W8"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 25.0;
		m_Wait = true;
	}
	
	//-----SHOTTING TUTORIAL-----------
	else if((*m_Save)["Dialog9"].asBool() == false)
	{
		(*m_Save)["Dialog9"] = true;
		m_Gui->ShowStoryText("Ok now that you know this, it is time to try some shooting.");
	}	
	else if((*m_Save)["Dialog10"].asBool() == false)
	{
		(*m_Save)["Dialog10"] = true;
		m_Gui->ShowStoryText("To fire your laser, simply press left mouse button.");
	}	
	else if((*m_Save)["Dialog10b"].asBool() == false)
	{
		(*m_Save)["Dialog10b"] = true;
		m_Gui->ShowStoryText("Try to shoot some meteorites.");
	}
	else if((*m_Save)["W10"].asBool() == false)
	{
		(*m_Save)["W10"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 60.0;
		m_Wait = true;
	}
	
	//-----REST OF TUTORIAL------------
	else if((*m_Save)["Dialog13"].asBool() == false)
	{
		(*m_Save)["Dialog13"] = true;
		m_Gui->ShowStoryText("If you forget anything that i just told you, you can try to locate emergency button \"F1\".");
	}
	else if((*m_Save)["Dialog14"].asBool() == false)
	{
		(*m_Save)["Dialog14"] = true;
		m_Gui->ShowStoryText("It is somewhere in your cockpit, so you can just press it.");
	}
	else if((*m_Save)["Dialog15"].asBool() == false)
	{
		(*m_Save)["Dialog15"] = true;
		m_Gui->ShowStoryText("Also pay attention to that green and blue indicators in the bottom of cockpit.");
	}
	else if((*m_Save)["Dialog16"].asBool() == false)
	{
		(*m_Save)["Dialog16"] = true;
		m_Gui->ShowStoryText("The blue one displays your current shield charge. Shield absorbs part of recieved damage.");
	}
	else if((*m_Save)["Dialog18"].asBool() == false)
	{
		(*m_Save)["Dialog18"] = true;
		m_Gui->ShowStoryText("The green one shows condition of the hull of your ship.");
	}
	else if((*m_Save)["Dialog19"].asBool() == false)
	{
		(*m_Save)["Dialog19"] = true;
		m_Gui->ShowStoryText("Once it is all red, you will die.\n   In space.\n      All alone.");
	}
	else if((*m_Save)["Dialog20"].asBool() == false)
	{
		(*m_Save)["Dialog20"] = true;
		m_Gui->ShowStoryText("The yellow one displays power of your engine,\nbut I bet you already know it.");
	}
	else if((*m_Save)["W20"].asBool() == false)
	{
		(*m_Save)["W20"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 15.0;
		m_Wait = true;
	}
	
	//-----JOB DESTROY SHIPS----------
	else if((*m_Save)["Dialog21"].asBool() == false)
	{
		(*m_Save)["Dialog21"] = true;
		m_Gui->ShowStoryText("Now that you are familiar enough with your ship, my boss has a job for you.");
	}
	else if((*m_Save)["Dialog22"].asBool() == false)
	{
		(*m_Save)["Dialog22"] = true;
		m_Gui->ShowStoryText("There are two old rusty abandoned ships nearby.");
	}
	else if((*m_Save)["Dialog23"].asBool() == false)
	{
		(*m_Save)["Dialog23"] = true;
		m_Gui->ShowStoryText("They are disturbing our navigation sattelites, but it costs credits to get them recycled.");
	}
	else if((*m_Save)["Dialog24"].asBool() == false)
	{
		(*m_Save)["Dialog24"] = true;
		m_Gui->ShowStoryText("Can you please locate those two ships and destroy them? Don't worry, we will pay you something.");
	}
	else if (m_Ship1->GetActive())
	{
		//Disable those in case that we are continuing previous story
		if((*m_Save)["Dialog25"].asBool() == true)
		{
			m_Ship1->SetActive(false);
			m_Ship2->SetActive(false);
			m_Ship1->ClearCollisionGeometry();
			m_Ship2->ClearCollisionGeometry();
		}

		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	else if (m_Ship2->GetActive())
	{
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	
	//-----TOWER 1 ACTIVE----------
	else if((*m_Save)["Dialog25"].asBool() == false)
	{
		(*m_Save)["Dialog25"] = true;
		m_Gui->ShowStoryText("Hm... This is weird, one of them had sent some kind of signal right before it exploded.");
	}
	else if((*m_Save)["Dialog26"].asBool() == false)
	{
		(*m_Save)["Dialog26"] = true;
		m_Gui->ShowStoryText("Oh no! It was some kind of virus.");
	}
	else if((*m_Save)["Dialog27"].asBool() == false)
	{
		(*m_Save)["Dialog27"] = true;
		m_Gui->ShowStoryText("It is hacking into our defensive turrets. Watch out!");
		m_Tower1->Enrage();
	}
	else if (m_Tower1->IsAlive())
	{
		if((*m_Save)["Dialog28"].asBool() == true)
		{
			m_Tower1->Model->SetActive(false);
			m_Tower1->Model->ClearCollisionGeometry();
		}
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	
	//-----TOWER 2,3 ACTIVE----------
	else if((*m_Save)["Dialog28"].asBool() == false)
	{
		(*m_Save)["Dialog28"] = true;
		m_Gui->ShowStoryText("Good, but it got into two others. Be carefull...");
		m_Tower2->Enrage();
		m_Tower3->Enrage();
	}
	else if (m_Tower2->IsAlive())
	{
		if((*m_Save)["Dialog29"].asBool() == true)
		{
			m_Tower2->Model->SetActive(false);
			m_Tower2->Model->ClearCollisionGeometry();
			m_Tower3->Model->SetActive(false);
			m_Tower3->Model->ClearCollisionGeometry();
		}
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	else if (m_Tower3->IsAlive())
	{
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}

	
	//-----TOWER 4,5,6 ACTIVE----------
	else if((*m_Save)["Dialog29"].asBool() == false)
	{
		(*m_Save)["Dialog29"] = true;
		m_Gui->ShowStoryText("The rest of the towers got hacked too... defend yourself.");
		m_Tower4->Enrage();
		m_Tower5->Enrage();
		m_Tower6->Enrage();
	}
	else if (m_Tower4->IsAlive())
	{
		if((*m_Save)["Dialog30"].asBool() == true)
		{
			m_Tower4->Model->SetActive(false);
			m_Tower4->Model->ClearCollisionGeometry();
			m_Tower5->Model->SetActive(false);
			m_Tower5->Model->ClearCollisionGeometry();
			m_Tower6->Model->SetActive(false);
			m_Tower6->Model->ClearCollisionGeometry();
		}
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	else if (m_Tower5->IsAlive())
	{
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	else if (m_Tower6->IsAlive())
	{
		m_Wait = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 2;
	}
	
	//------ENDING----------------
	else if((*m_Save)["Dialog30"].asBool() == false)
	{
		(*m_Save)["Dialog30"] = true;
		m_Gui->ShowStoryText("Ufff... that was close");
	}
	else if((*m_Save)["Dialog31"].asBool() == false)
	{
		(*m_Save)["Dialog31"] = true;
		m_Gui->ShowStoryText("Good thing is that we are alive, but my boss won't be happy.");
	}
	else if((*m_Save)["Dialog32"].asBool() == false)
	{
		(*m_Save)["Dialog32"] = true;
		m_Gui->ShowStoryText("I think I cant pay you your credits now. Sorry...");
	}
	else if((*m_Save)["W32"].asBool() == false)
	{
		(*m_Save)["W32"] = true;
		m_WaitTime = dtCore::System::GetInstance().GetSimulationTime() + 15.0;
		m_Wait = true;
	}
	else if((*m_Save)["Dialog33"].asBool() == false)
	{
		(*m_Save)["Dialog33"] = true;
		m_Gui->ShowStoryText("Congratulations'\nYou have just finished campaign. Feel free to explore rest of the universe.");
	}
}

///Called every game update to check timed delay 
void Level::Update()
{
	if(PlayerDead == true)
		return;

	if (m_Wait == true && m_WaitTime < dtCore::System::GetInstance().GetSimulationTime())
	{
		m_Wait = false;
		Advance();
	}
	
	//Update towers
	dtCore::Scene *sc = dtABC::Application::GetInstance(0)->GetScene();
	double simTime = dtCore::System::GetInstance().GetSimulationTime();
	m_Tower1->Update(m_Player, sc, simTime);
	m_Tower2->Update(m_Player, sc, simTime);
	m_Tower3->Update(m_Player, sc, simTime);
	m_Tower4->Update(m_Player, sc, simTime);
	m_Tower5->Update(m_Player, sc, simTime);
	m_Tower6->Update(m_Player, sc, simTime);
}

///Checks map and if it finds players ship, then returns it
dtCore::RefPtr<dtCore::Object> Level::GetPlayer(dtCore::Map &map)
{	
	typedef std::vector<dtCore::RefPtr<dtCore::ActorProxy>> ProxyVector;
	ProxyVector proxies;
	map.FindProxies(proxies, "*");
	ProxyVector::iterator end_iter = proxies.end();

	// process the proxies
	for (ProxyVector::iterator iter = proxies.begin(); iter != end_iter; ++iter)
	{
		if ((*iter)->GetName() == "Player")
		{
			m_Player = dynamic_cast<dtCore::Object*>((*iter)->GetActor());

			m_Player->EnableDynamics(true);
			m_Player->GetBodyWrapper()->SetLinearDamping(0.005f);
			m_Player->GetBodyWrapper()->SetAngularDamping(0.02f);
			
			dMass *mass = new dMass();	
			m_Player->GetMass(mass);		
			float inert = mass->mass * mass->mass;
			mass->setParameters(mass->mass,
				mass->c[0], mass->c[1], mass->c[2],
				mass->mass, mass->mass, mass->mass,
				0.0f, 0.0f, 0.0f); 
			m_Player->SetMass(mass);

		}
	}
	return m_Player;
}

///Checks map and if it finds players shield, then returns it
dtCore::RefPtr<dtCore::Object> Level::GetPlayerShield(dtCore::Map &map)
{
	dtCore::RefPtr<dtCore::Object> shield;
	
	typedef std::vector<dtCore::RefPtr<dtCore::ActorProxy>> ProxyVector;
	ProxyVector proxies;
	map.FindProxies(proxies, "*");
	ProxyVector::iterator end_iter = proxies.end();

	// process the proxies
	for (ProxyVector::iterator iter = proxies.begin(); iter != end_iter; ++iter)
	{
		if ((*iter)->GetName() == "PlayerShield")
		{
			shield = dynamic_cast<dtCore::Object*>((*iter)->GetActor());
		}
	}
	return shield;
}

///Checks map and if it finds players ships engine particle, then returns it
dtCore::RefPtr<dtCore::ParticleSystem> Level::GetPlayerEngineParticle(dtCore::Map &map)
{
	dtCore::RefPtr<dtCore::ParticleSystem> particle;
	
	typedef std::vector<dtCore::RefPtr<dtCore::ActorProxy>> ProxyVector;
	ProxyVector proxies;
	map.FindProxies(proxies, "*");
	ProxyVector::iterator end_iter = proxies.end();

	// process the proxies
	for (ProxyVector::iterator iter = proxies.begin(); iter != end_iter; ++iter)
	{
		if ((*iter)->GetName() == "PlayerEngineParticle")
		{
			particle = dynamic_cast<dtCore::ParticleSystem*>((*iter)->GetActor());
		}
	}
	return particle;
}

///Load all towers from current level
void Level::LoadLevelObjects(dtCore::Map &map)
{	
	dtCore::RefPtr<dtCore::Object> temp;
	
	typedef std::vector<dtCore::RefPtr<dtCore::ActorProxy>> ProxyVector;
	ProxyVector proxies;
	map.FindProxies(proxies, "D_*");
	ProxyVector::iterator end_iter = proxies.end();

	for (ProxyVector::iterator iter = proxies.begin(); iter != end_iter; ++iter)
	{
		if ((*iter)->GetName() == "D_Tower_1")
		{
			m_Tower1 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Tower_2")
		{
			m_Tower2 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Tower_3")
		{
			m_Tower3 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Tower_4")
		{
			m_Tower4 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Tower_5")
		{
			m_Tower5 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Tower_6")
		{
			m_Tower6 = new Tower(dynamic_cast<dtCore::Object*>((*iter)->GetActor()));
		}
		else if ((*iter)->GetName() == "D_Ship_1")
		{
			m_Ship1 = dynamic_cast<dtCore::Object*>((*iter)->GetActor());
		}
		else if ((*iter)->GetName() == "D_Ship_2")
		{
			m_Ship2 = dynamic_cast<dtCore::Object*>((*iter)->GetActor());
		}
	}
}

///Resets level after new game
void Level::Reset()
{
	m_WaitTime = 0;
	m_Wait = false;
}