/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#pragma once
#define dSINGLE //resolves fatal error C1004: unexpected end-of-file found

#include <cstdio>
#include <iostream>

#include <dtABC/application.h>
#include <dtCore/effectmanager.h>
#include <dtCore/system.h>
#include <dtCore/refptr.h>
#include <dtCore/object.h>
#include <dtCore/transform.h>
#include <dtCore/mouse.h>
#include <dtCore/odebodywrap.h>
#include <dtCore/base.h>
#include <dtCore/particlesystem.h>
#include <math.h>

#include "sound.h"

#include "json/json-forwards.h"
#include "json/json.h"


class Player: public dtCore::Base
{
//Declarations
public:
	enum DamageType
	{
		UNKNOWN,
		COLLISION,
		LASER_SHOT,
	};

//Variables
public:
	double								   Hull;
	double								   Shield;
	double							       EnginePower;
	
	bool							       InvertX;
	bool								   InvertY;

	dtCore::RefPtr<dtCore::Object>         Model;
	dtCore::RefPtr<dtCore::Object>         ShieldModel;
	dtCore::RefPtr<dtCore::ParticleSystem> EngineParticle;

private:
	bool   m_MoveLeft;
	bool   m_MoveRight;
	bool   m_MoveUp;
	bool   m_MoveDown;
	bool   m_RotateLeft;
	bool   m_RotateRight;
	float  m_MouseHorizontalMovement;
	float  m_MouseVerticalMovement;
	double m_LastLaserShot;
	double m_ShieldVisible;

//Methods
public:
	Player::Player();
	void Player::Damage(double value, Player::DamageType damageType);
	void Player::SetModel(dtCore::RefPtr<dtCore::Object> playerModel);
	void Player::Update(const double deltaSimTime);
	void Player::KeyChanged(const int key, const bool value);
	void Player::MouseMoved(float horizontal, float vertical);
	void Player::MousePressed(dtCore::Mouse::MouseButton button);
	void Player::ProcessMovement();

private:
	
	void Player::MoveShield();
	void Player::MoveItems();
	void Player::UpdateEngineParticle();
};