/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "shooter.h"

Shooter*      Shooter::m_Instance     = NULL;

///Constructor
Shooter::Shooter()
{
	m_Scene = dtABC::Application::GetInstance(0)->GetScene();
}

///Destructor
Shooter::~Shooter(void)
{
	if(m_Instance != NULL)
	{
		//Clear reference
		m_Instance = NULL;
	}
}

///Handles incoming messages
void Shooter::OnMessage(MessageData* data)
{
	//Message to shoot lasers
	if(data->message == "ShootLaser")
	{
		dtCore::Transform *trans = static_cast<dtCore::Transform*>( data->userData );
		ShootLaser(trans);
	}
	//Physics update message
	else if (data->message == "physics_step")
	{
		CheckLasers();
	}
}

///Shoots laser in direction given by transform
void Shooter::ShootLaser(dtCore::Transform *trans, float dispersion)
{
	if(trans == 0)
		return;

	//Load model
	LaserShot shot;
	shot.CreationTime = dtCore::System::GetInstance().GetSimulationTime();
	shot.Model = new dtCore::Object("LaserShot");
	shot.Model->LoadFile("gamedata/STAGEMap/StaticMeshes/laser.osgb");

	//Get position and rotation
	osg::Vec3 position;
	osg::Quat rotation;
	trans->Get(position, rotation);

	//Put shot before model
	dtCore::Transform positionBeforeObject;
	positionBeforeObject.Set(position + (rotation * osg::Vec3(dispersion, 15, 0)), rotation);

	//Set shot parameters
	shot.Model->SetTransform(positionBeforeObject);
	shot.Model->SetCollisionBox();
	shot.Model->SetMass(0.01f);
	shot.Model->SetName("LaserShot");
	shot.Model->EnableDynamics();
	shot.Model->GetBodyWrapper()->SetLinearDamping(0.001f);
	osg::Vec3 speed(0.0, 300.0, 0.0);
	
	//Add shot to scene and to buffer
	LaserShotBuffer.push_front(shot);
	m_Scene->AddChild(shot.Model.get());

	//Shoot shot
	shot.Model->GetBodyWrapper()->ApplyForce((rotation*speed));
}

///Checks existing lasers and removes those who are living too long
void Shooter::CheckLasers()
{
	double currTime = dtCore::System::GetInstance().GetSimulationTime();

	while(LaserShotBuffer.size() > 0 && LaserShotBuffer.back().CreationTime + 1.5 < currTime)
	{
		m_Scene->RemoveChild(LaserShotBuffer.back().Model.get());
		LaserShotBuffer.pop_back();
	}
}

///The only point which may be used to call shooter manager
Shooter * Shooter::GetInstance()
{
	if(m_Instance == NULL)
	{
		m_Instance = new Shooter();
	}
	return m_Instance;
}