/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "sound.h"


const char* Sound::MenuButtonPath = "gamedata/Sounds/menu.wav";
const char* Sound::LaserShotPath  = "gamedata/Sounds/laser.wav";
const char* Sound::ExplosionPath  = "gamedata/Sounds/explosion.wav";
Sound*      Sound::m_Instance     = NULL;

//Constructor
Sound::Sound()
{
	//Initialize audio
	dtAudio::AudioManager::Instantiate();

	//Preload audio files
	dtAudio::AudioManager::GetInstance().LoadFile(LaserShotPath);
	m_SoundLaser = dtAudio::AudioManager::GetInstance().NewSound();
	m_SoundLaser->LoadFile(LaserShotPath);
	m_SoundLaser->SetStopCallback(Sound::OnSoundStopped, this);
	
	dtAudio::AudioManager::GetInstance().LoadFile(MenuButtonPath);
	m_SoundMenu = dtAudio::AudioManager::GetInstance().NewSound();
	m_SoundMenu->LoadFile(MenuButtonPath);
	m_SoundMenu->SetStopCallback(Sound::OnSoundStopped, this);
	
	dtAudio::AudioManager::GetInstance().LoadFile(ExplosionPath);
	m_SoundExplosion = dtAudio::AudioManager::GetInstance().NewSound();
	m_SoundExplosion->LoadFile(ExplosionPath);
	m_SoundExplosion->SetStopCallback(Sound::OnSoundStopped, this);
}

///Destructor
Sound::~Sound(void)
{
	if(m_Instance != NULL)
	{
		//Free sounds
		dtAudio::AudioManager::GetInstance().FreeSound(m_SoundLaser);
		dtAudio::AudioManager::GetInstance().FreeSound(m_SoundMenu);
		dtAudio::AudioManager::GetInstance().FreeSound(m_SoundExplosion);
		//Kill audio
		dtAudio::AudioManager::Destroy();
		//Clear reference
		m_Instance = NULL;
	}
}

///The only point which may be used to call sound manager
Sound * Sound::GetInstance()
{
	if(m_Instance == NULL)
	{
		m_Instance = new Sound();
	}
	return m_Instance;
}

///Calback that frees sound which just stopped palying
void Sound::OnSoundStopped(dtAudio::Sound* sound, void* param)
{
   // Free only sounds not defined in this class
   if (sound == static_cast<Sound*>(param)->m_SoundLaser)
   {
      return;
   }
   if (sound == static_cast<Sound*>(param)->m_SoundMenu)
   {
      return;
   }
   if (sound == static_cast<Sound*>(param)->m_SoundExplosion)
   {
      return;
   }
   dtAudio::AudioManager::GetInstance().FreeSound(sound);
}

/// Play sound of laser being shot
dtAudio::Sound* Sound::PlayLaserShot()
{
	dtAudio::AudioManager &mgr = dtAudio::AudioManager::GetInstance();
	dtAudio::Sound* sound = mgr.NewSound();
	sound->LoadFile(LaserShotPath);
	sound->SetStopCallback(OnSoundStopped, m_Instance);
	sound->Play();
	sound->SetRolloffFactor(0.05f);
	return sound;
}

/// Play sound of pressed button
void Sound::PlayMenuButton()
{
	dtAudio::AudioManager &mgr = dtAudio::AudioManager::GetInstance();
	dtAudio::Sound* sound = mgr.NewSound();
	sound->LoadFile(MenuButtonPath);
	sound->SetStopCallback(OnSoundStopped, m_Instance);
	sound->Play();
	sound->SetRolloffFactor(0);
}

/// Play sound of explosion
dtAudio::Sound* Sound::PlayExplosion()
{
	dtAudio::AudioManager &mgr = dtAudio::AudioManager::GetInstance();
	dtAudio::Sound* sound = mgr.NewSound();
	sound->LoadFile(ExplosionPath);
	sound->SetStopCallback(OnSoundStopped, m_Instance);
	sound->Play();
	sound->SetRolloffFactor(0.01f);
	return sound;
}
