/*****************************************************************************\
 *
 * Author: Sauer Vaclav
 * Email:  xsauer00<at>stud.fit.vutbr.cz
 *
 * Bachelors's thesis: Space Game Project Using Delta3D Simulation Engine 2014
 * 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "tower.h"

///Constructor
Tower::Tower(dtCore::Object *model)
{
	Model = model;
	m_NextShotTime = 15;
	m_Enraged = false;
}

///Update of tower
void Tower::Update(dtCore::RefPtr<dtCore::Object> m_Player, dtCore::Scene *sc, double simTime)
{
	//Some checks
	if(m_Enraged == false)
		return;
	if(Model == NULL)
		return;
	if(m_Player == NULL)
		return;
	if(sc == NULL)
		return;
	//return if tower is idle
	if(Model->GetActive() == false)
		return;

	//Get transforms of player and tower
	dtCore::Transform transPlayer;
	dtCore::Transform transTower;
	Model->GetTransform(transTower);
	m_Player->GetTransform(transPlayer);

	//Check if tower is close enough to shoot player
	if( transTower.CalcDistance(transPlayer) > 900)
		return;
	
	//Rotate tower to player
	transTower.Set(transTower.GetTranslation(), transPlayer.GetTranslation(), transTower.GetUpVector());
	Model->SetTransform(transTower);

	//Shoot lasers
	if(simTime > m_NextShotTime)
	{
		m_NextShotTime = m_ShootSide ? (simTime +  (rand() % 20 + 50) * 0.001) : (simTime +  (rand() % 20 + 260) * 0.001);

		//Toggle left and right cannon
		m_ShootSide = !m_ShootSide;

		Shooter::GetInstance()->ShootLaser(&transTower, m_ShootSide ? -7 : 7);
		Sound::GetInstance()->PlayLaserShot()->SetPosition(transTower.GetTranslation());
	}
}

///Tower enters into defensive mode, shooting player
void Tower::Enrage()
{
	m_Enraged = true;
}

///Check if tower is still alive
bool Tower::IsAlive()
{
	return Model->GetActive();
}
